==> w1druppel-0.1.7.tgz: package/src/auth.c <==
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <sha33/sha33.h>

#include "drup/types.h"
#include "private/auth.h"
#include "private/ds1961.h"
#include "private/ds2482.h"
#include "private/util.h"

void make_challenge (uint8_t * const buf) {
  srandom (time (NULL));
  for (int i = 0; i < 3; i++)
    buf [i] = random ();
}

/* Doesn't always set errno.
 */
static bool read_with_challenge (const struct key * const key, const uint8_t * const challenge, uint8_t * const auth_page_read, uint8_t * const rmac) {
  uint8_t data[8] = {
    0, 0, 0, 0, challenge[0], challenge[1], challenge[2], 0,
  };
  try (ds1961_write_scratchpad, (key, key->auth_addr, data))
  try_with (ds1961_read_auth_page, (key, auth_page_read, rmac), ac_warn_rf)
  return true;
}

static void calc_lmac (uint16_t auth_addr, uint64_t device_id, const uint8_t * const challenge, const uint8_t * const secret8, const uint8_t * const auth_page_read, uint8_t * const lmac) {
  uint32_t hash_input[16];
  uint32_t hash_output[16];
  uint8_t id_input[7];

  for (int i = 0; i < 7; i++)
    id_input[i] = (device_id & (0xFFLL << i * 8)) >> i * 8;

  hash_input[0] = (secret8[0] << 24) | (secret8[1] << 16) | (secret8[2] << 8) | secret8[3];

  for (int i = 0; i < 32; i += 4)
    hash_input[i/4 + 1] = (auth_page_read[i] << 24) | (auth_page_read[i + 1] << 16) | (auth_page_read[i + 2] << 8) | auth_page_read[i + 3];

  hash_input[9] = 0xFFFFFFFF;
  uint8_t mp = (0b1000 << 3) | ((auth_addr >> 5) & 0b111);
  hash_input[10] = (mp << 24) | (id_input[0] << 16) | (id_input[1] << 8) | id_input[2];
  hash_input[11] = (id_input[3] << 24) | (id_input[4] << 16) | (id_input[5] << 8) | id_input[6];
  hash_input[12] = (secret8[4] << 24) | (secret8[5] << 16) | (secret8[6] << 8) | secret8[7];
  hash_input[13] = (challenge[0] << 24) | (challenge[1] << 16) | (challenge[2] << 8) | 0x80;
  hash_input[14] = 0;
  hash_input[15] = 0x1B8;

  sha33_compute_sha_vm_32 (hash_input, hash_output);
  sha33_hash_to_mac (hash_output, lmac);
}

/* Doesn't always set errno.
 */
bool auth_check (const struct drup_state * const state, const struct key * const key, bool * const authenticated) {
  uint16_t const auth_addr = key->auth_addr;
  uint64_t const device_id = key->device_id;

  uint8_t challenge[3];
  make_challenge (challenge);

  // --- read during calculation of rmac.
  uint8_t auth_page_read[32];
  // --- calculated by device.
  uint8_t rmac[20];
  // --- calculated by us.
  uint8_t lmac[20];

  try (ds2482_set_overdrive, (state->master, true))
  try_with (read_with_challenge, (key, challenge, auth_page_read, rmac), ac_warn_rf)

  uint8_t secret8[8];
  const uint8_t * const secret16 = key->secret;
  secret_16_to_8 (secret16, secret8);
  calc_lmac (auth_addr, device_id, challenge, secret8, auth_page_read, lmac);
  *authenticated = memcmp (lmac, rmac, sizeof rmac) == 0;
  return true;
}

==> w1druppel-0.2.3.tgz: package/src/auth.c <==
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <sha33/sha33.h>

#include "drup/types.h"
#include "private/auth.h"
#include "private/ds1961.h"
#include "private/ds2482.h"
#include "private/util.h"

void make_challenge (uint8_t * const buf) {
  srandom (time (NULL));
  for (int i = 0; i < 3; i++)
    buf [i] = random ();
}

/* Doesn't always set errno.
 */
static bool read_with_challenge (const struct key * const key, const uint8_t * const challenge, uint8_t * const auth_page_read, uint8_t * const rmac) {
  uint8_t data[8] = {
    0, 0, 0, 0, challenge[0], challenge[1], challenge[2], 0,
  };
  try (ds1961_write_scratchpad, (key, key->auth_addr, data))
  try_with (ds1961_read_auth_page, (key, auth_page_read, rmac), ac_warn_rf)
  return true;
}

static void calc_lmac (uint16_t auth_addr, uint64_t device_id, const uint8_t * const challenge, const uint8_t * const secret8, const uint8_t * const auth_page_read, uint8_t * const lmac) {
  uint32_t hash_input[16];
  uint32_t hash_output[16];
  uint8_t id_input[7];

  for (int i = 0; i < 7; i++)
    id_input[i] = (device_id & (0xFFLL << i * 8)) >> i * 8;

  hash_input[0] = (secret8[0] << 24) | (secret8[1] << 16) | (secret8[2] << 8) | secret8[3];

  for (int i = 0; i < 32; i += 4)
    hash_input[i/4 + 1] = (auth_page_read[i] << 24) | (auth_page_read[i + 1] << 16) | (auth_page_read[i + 2] << 8) | auth_page_read[i + 3];

  hash_input[9] = 0xFFFFFFFF;
  uint8_t mp = (0b1000 << 3) | ((auth_addr >> 5) & 0b111);
  hash_input[10] = (mp << 24) | (id_input[0] << 16) | (id_input[1] << 8) | id_input[2];
  hash_input[11] = (id_input[3] << 24) | (id_input[4] << 16) | (id_input[5] << 8) | id_input[6];
  hash_input[12] = (secret8[4] << 24) | (secret8[5] << 16) | (secret8[6] << 8) | secret8[7];
  hash_input[13] = (challenge[0] << 24) | (challenge[1] << 16) | (challenge[2] << 8) | 0x80;
  hash_input[14] = 0;
  hash_input[15] = 0x1B8;

  sha33_compute_sha_vm_32 (hash_input, hash_output);
  sha33_hash_to_mac (hash_output, lmac);
}

/* Doesn't always set errno.
 */
bool auth_check (const struct drup_state * const state, const struct key * const key, uint8_t cfg, bool * const authenticated) {
  uint16_t const auth_addr = key->auth_addr;
  uint64_t const device_id = key->device_id;

  uint8_t challenge[3];
  make_challenge (challenge);

  // --- read during calculation of rmac.
  uint8_t auth_page_read[32];
  // --- calculated by device.
  uint8_t rmac[20];
  // --- calculated by us.
  uint8_t lmac[20];

  try (ds2482_write_configuration, (state->master, cfg))
  try_with (read_with_challenge, (key, challenge, auth_page_read, rmac), ac_warn_rf)

  uint8_t secret8[8];
  const uint8_t * const secret16 = key->secret;
  secret_16_to_8 (secret16, secret8);
  calc_lmac (auth_addr, device_id, challenge, secret8, auth_page_read, lmac);
  *authenticated = memcmp (lmac, rmac, sizeof rmac) == 0;
  return true;
}
